#!/bin/sh
#
# $Id: bin/ipv6Manager.sh 1.11 2007/11/16 10:22:05MST Chris Cason (ccason) dev  $
##

## Path to application home
#-- applicationHome="/home/embedded/library"
## Source include file.
#-- . ${applicationHome}/AppManager/bin/include.sh
.  /home/embedded/library/AppManager/bin/include.sh
.  /home/embedded/library/AppManager/bin/serviceManager.sh

eth2interfaceConfigurationFile="/etc/network/interfaceEth2"
statelessAutoConfigurationFile="/etc/disableStatelessAutoConfiguration"
STATIC_ADDRESS_INFO=""
EXTERNAL_INTERFACE="eth2"

## Error Codes:
EXIT_STATUS_WRONG_FORMAT=5
EXIT_STATUS_NO_DHCP_ADDRESS=32


#########################################
# getStaticAddressInfo
#
# Inputs: None
# Output: Populates the variable STATIC_ADDRESS_INFO with the static IP,
# prefix and gateway found in the file /etc/network/interfaceEth2. It has the
# following format "2001:db8:0:f101::1001/64?2001:db8:0:f101::1005"
#
#########################################
getStaticAddressInfo()
{
   STATIC_ADDRESS_INFO=`cat $eth2interfaceConfigurationFile | awk ' 
   BEGIN { startProcessing=0; staticIP=""; prefix=""; gateway=""; }
   {
      inet6=$3
      static=$4

      if( $3 == "inet6" && $4 == "static" ) 
      {
         startProcessing=1
         next
      }

      if( startProcessing == 1 ) 
      {
         staticIP=$2
         startProcessing++;
         next
      }
      if( startProcessing == 2 ) 
      {
         prefix=$2
         startProcessing++;
         next
      }
      if( startProcessing == 3 ) 
      {
         gateway=$2
         startProcessing++;
      }
   } END { printf("%s/%s?%s",staticIP, prefix, gateway) }'`
   COMMAND_EXIT_STATUS=$?
   if [ $COMMAND_EXIT_STATUS -eq 0 ]; then
      LogTrace "Get Static Address Information: $STATIC_ADDRESS_INFO"
      if [ "$STATIC_ADDRESS_INFO" != "/?" ]; then
         echo "$STATIC_ADDRESS_INFO"
      fi
   else
      LogError "Failed to get Static Address Information: $STATIC_ADDRESS_INFO"
   fi
}

#########################################
# deleteStaticAddress
# Deletes the static IP information from the "/etc/network/interfaceEth2"
# file. This includes the address, gateway and prefix mask.
#
# Inputs: flag to restart the network interface
#########################################
deleteStaticAddress()
{
   LogTrace "Delete Static Address"
   local tmpFile="/tmp/staticIPfile.$$"
   local restart="$1"
   local address=`grep 'address.*:.*' /etc/network/interfaceEth2 | awk '{print $2}'`

   if [ "X${address}" != "X" ]; then
      address=`ifconfig eth2 | grep " ${address}/" | awk '{print $3}' 2>/dev/null`
   fi

   if [ $? -ne 0 ]; then
      address="";
   fi

   if [ -f "$tmpFile" ]; then
      RunCommand "rm $tmpFile"
   fi
   RunCommand "touch $tmpFile"
   cat $eth2interfaceConfigurationFile | awk ' 
   { 
      inet6=$3
      static=$4

      if( $3 == "inet6" && $4 == "static" ) 
      {
         startProcessing=1
         next
      }

      if( !(startProcessing > 0 && startProcessing < 4) ) 
      {
         printf("%s\n", $0);
         next
      }
      else if( startProcessing == 1 ) 
      {
         startProcessing++;
         next
      }
      else if( startProcessing == 2 ) 
      {
         startProcessing++;
         next
      }
      else if( startProcessing == 3 ) 
      {
         startProcessing++;
      }
   } ' >> $tmpFile
   COMMAND_EXIT_STATUS=$?
   if [ $COMMAND_EXIT_STATUS -eq 0 ]; then
      RunCommand "mv $tmpFile $eth2interfaceConfigurationFile";
      if [ "X${address}" != "X" ]; then
         RunCommand "ip addr del $address dev eth2"
      fi
      if [ "X$restart" != "X" ]; then
         restartNetwork
      fi
   fi
}

#########################################
# getAllAddresses
#
# Output: Returns all the IP addresses configured on eth2 interface.
#########################################
getAllAddresses()
{
   LogTrace "Get all addresses"
   ALL_ADDRESS_INFO=`ifconfig $EXTERNAL_INTERFACE | awk ' 
   BEGIN { startProcessing=0; addresses=""; }
   {
      inet6=$3
      static=$4

      if( $1 == "inet6" && $2 == "addr:" ) 
      {
         if( startProcessing == 0 )
         {
            printf("%s", $3);
         }
         else
         {
            printf("?%s", $3);
         }
         startProcessing++
         next
      }

   } '`
   COMMAND_EXIT_STATUS=$?
   if [ $COMMAND_EXIT_STATUS -eq 0 ]; then
      LogTrace "Addresses: $ALL_ADDRESS_INFO"
      echo $ALL_ADDRESS_INFO
   fi
}

#########################################
# configureStaticAddress
# Configure Static IP. This deletes the existing entry in eth2 configuration
# file and then adds the new entry
#
# Inputs: New IP, gateway.
#########################################
configureStaticAddress()
{
   local staticPrefixGateway="$1"
   local addressWithPrefix=""
   local address=""
   local prefix=""
   local gateway=""
   
   if [ "$staticPrefixGateway" = "delete" ]; then
      deleteStaticAddress
      restartNetwork
   else
      addressWithPrefix=`echo $staticPrefixGateway | awk -F? '{print $1}'`
      address=`echo $addressWithPrefix | awk -F/ '{print $1}'`
      prefix=`echo $addressWithPrefix | awk -F/ '{print $2}'`
      gateway=`echo $staticPrefixGateway | awk -F? '{print $2}'`

      LogTrace "Configure Static Address - address: $address, prefix: $prefix, gateway: $gateway"
      if [ "X$address" = "X" -o "X$prefix" = "X" -o "X$gateway" = "X" ]; then
         LogError "Invalid format"
         COMMAND_EXIT_STATUS=$EXIT_STATUS_WRONG_FORMAT
      fi

      if [ $COMMAND_EXIT_STATUS -eq 0 ]; then
         deleteStaticAddress
         if [ $COMMAND_EXIT_STATUS -eq 0 ]; then
            echo "iface eth2 inet6 static" >> $eth2interfaceConfigurationFile
            echo "     address $address" >> $eth2interfaceConfigurationFile
            echo "     netmask $prefix" >> $eth2interfaceConfigurationFile
            echo "     gateway $gateway" >> $eth2interfaceConfigurationFile
            restartNetwork
         fi
      fi
   fi
}

#########################################
# checkForAddress
#
# Verify interface got an address
#########################################
checkForAddress()
{
   local timeout=0
   while [ 1 ]; do
      if [ -f "/etc/ipv6established" ]; then
         COMMAND_EXIT_STATUS=0
         break
      else
         sleep 1
         timeout=`expr $timeout + 1`
         if [ $timeout -gt 120 ]; then
            LogTrace "Failed to get ipv6 address for interface eth2"
            COMMAND_EXIT_STATUS=$EXIT_STATUS_NO_DHCP_ADDRESS
            break
         fi
      fi
   done
}

#########################################
# enableDHCP
# Enable/Disable DHCP.
#
# Inputs: "on" or "off"
#########################################
enableDHCP()
{
   local option="$1"
   LogTrace "Enable DHCP: $option"
   if [ "$option" = "on" ]; then
      if [ ! -f "/etc/ipv6dhcp" ]; then
         RunCommand "touch /etc/ipv6dhcp"
      fi
      RunCommand "rm -f /etc/ipv6established"
      restartNetwork
      checkForAddress
   elif [ "$option" = "off" ]; then
      if [ -f "/etc/ipv6dhcp" ]; then
         RunCommand "rm /etc/ipv6dhcp"
      fi
      restartNetwork
   else
      Usage
   fi
}

#########################################
# isStatelessEnabled
# Check to see if statless auto configuration is enabled
#
#########################################
isStatelessEnabled()
{
   local state=`cat /proc/sys/net/ipv6/conf/eth2/autoconf`
   LogTrace "Report stateless auto configuration"
   echo $state
}

#########################################
# enableStateless
# enable/disable statless auto configuration
#
# Input: 1 enable or 0 for disable
#
#########################################
enableStateless()
{
   local newState="$1"
   RunCommand "echo $newState > /proc/sys/net/ipv6/conf/eth2/autoconf"
   if [ "$newState" = "0" ]; then
      LogTrace "Disable stateless auto configuration"
      if [ ! -f "$statelessAutoConfigurationFile" ]; then
         RunCommand "touch $statelessAutoConfigurationFile"
      else
         LogError "Stateless auto configuration file \"$statelessAutoConfigurationFile\" already exist."
      fi
   else
      LogTrace "Enable stateless auto configuration"
      if [ -f "$statelessAutoConfigurationFile" ]; then
         RunCommand "rm $statelessAutoConfigurationFile"
      else
         LogError "Stateless auto configuration file \"$statelessAutoConfigurationFile\" does not exist."
      fi
   fi
   deleteAddresses
   restartNetwork
}

#########################################
# deleteAddresses
# Delete all ipv6 address except link local
#
#########################################
deleteAddresses()
{
   local addresses=`ifconfig eth2 | grep 'inet6 addr:' | grep -v 'Scope:Link' | awk '{print $3}'`
   local address=""

   for address in `echo $addresses`
   do
      RunCommand "ip addr del $address dev eth2"
   done
}

#########################################
# displayConfigReport
#
# Display the current configuration
# in a format suitable for the 
# configuration report
#########################################
displayConfigReport()
{
    if [ -f "/home/embedded/library/CoreService/dat/ipv6Enabled" ]; then
       if [ -f "/etc/ipv6dhcp" ]; then
          echo "IPv6 DHCP: enabled"
       else
          echo "IPv6 DHCP: disabled"
       fi

      ALL_ADDRESS_INFO=`ifconfig $EXTERNAL_INTERFACE | awk ' 
      BEGIN { startProcessing=0; addresses=""; }
      {
        inet6=$3
        static=$4
   
        if( $1 == "inet6" && $2 == "addr:" ) 
        {
            if( startProcessing == 0 )
            {
                printf("IPv6 Addresses: %s", $3);
            }
            else
            {
                printf(", %s", $3);
            }
            startProcessing++
            next
         }
   
      } '`
      COMMAND_EXIT_STATUS=$?
      if [ $COMMAND_EXIT_STATUS -eq 0 ]; then
           echo $ALL_ADDRESS_INFO
      fi
      grep 'iface eth2 inet6 static' /etc/network/interfaceEth2 >/dev/null 2>&1
      if [ $? -eq 0 ]; then
         STATIC_ADDRESS_INFO=`cat $eth2interfaceConfigurationFile | awk ' 
         BEGIN { startProcessing=0; staticIP=""; prefix=""; gateway=""; }
         {
            inet6=$3
            static=$4
      
            if( $3 == "inet6" && $4 == "static" ) 
            {
               startProcessing=1
               next
            }
      
            if( startProcessing == 1 ) 
            {
               staticIP=$2
               startProcessing++;
               next
            }
            if( startProcessing == 2 ) 
            {
               prefix=$2
               startProcessing++;
               next
            }
            if( startProcessing == 3 ) 
            {
               gateway=$2
               startProcessing++;
            }
         } END { printf("IPv6 Static IP: %s/%s\nIPv6 Gateway: %s\n",staticIP, prefix, gateway) }'`
         COMMAND_EXIT_STATUS=$?
         if [ $COMMAND_EXIT_STATUS -eq 0 ]; then
             echo "$STATIC_ADDRESS_INFO"
         fi
      fi
  fi
}

#########################################
# Display usage message
#
# Output: Display usage message and exit the script.
#########################################
Usage()
{
   echo "Usage: $0 < Action > [ Options ]
   Actions:

     -c             : configure IP address, requires a -s or -d option

     -g \"option\"    : get information, where \"option\" is one of the following

        static - get static address info
        all    - get all address configured
        host   - get the library hostname
        dns    - get the DNS servers
        ntp    - get the NTP servers

     -h             : show this message

     -l             : check if stateless auto configuration is enabled(1)/disabled(0)

     -n \"hostname\"  : change library hostname, requires a hosname argument
      
     -r             : remove static address

     -D \"servers\"   : set the DNS addresses with the following format

        <server 1>[ <server 2> ...]

                      retrieve DNS with \"status\"
                    
     -t \"servers\"   : set the NTP time servers with the following format

        <server 1>[ <server 2> ...]

                      retrieve NTP Servers with \"status\"
                      disable with \"disable\"

     -L \"enable(1)/disable(0)\" : enable/disable stateless auto configuration.

     -E              : Display the configuration report.

   Options:

     -d \"state\"     : dhcp, states are \"on\", \"off\" or  \"status\"

     -s             : configure Static address (used with -c), where static
                      address, netmask and gateway are provide in the following format

        2001:db8:0:f101::1001/64?2001:db8:0:f101::1005 

        notice that the \"?\" delimites the IP from the netmask and gateway.

        To delete this address -s \"delete\"

   Examples:

      configure static IP 2001:db8:0:f101::1001/64 with gateway 2001:db8:0:f101::1005
         $0 -c -s \"2001:db8:0:f101::1001/64?2001:db8:0:f101::1005\"

      enable dhcp
         $0 -c -d \"on\"

      get static address info
         $0 -g static

      set ntp servers
         $0 -t "2001::1 172.188.22.1"
   ";
   exit 1
}

#################################################################################
####                               M A I N                                   ####
#################################################################################

operation=""
getAddress=""
removeStaticAddress=""
configureAddress=""
configureDHCP=""
changeHostname=""
newHostname=""
reportStatelessStatus=""
changeStatelessStatus=""
configReport=""


if [ $# -eq 0 ]; then
   Usage
else
   while getopts "cd:g:hln:rs:D:t:L:E" opt
   do
      case "$opt" in
         c) configureAddress="true"
         ;;
         d) configureDHCP="${OPTARG}"
         ;;
         h) Usage
         ;;
         g) operation="${OPTARG}"
         getAddress="true"
         ;;
         l) reportStatelessStatus="true"
         ;;
         n) newHostname="${OPTARG}"
         changeHostname="true"
         ;;
         r) removeStaticAddress="true"
         ;;
         s) staticAddress="${OPTARG}"
         ;;
         t) ntpServers="${OPTARG}"
         ;;
         D) domainServers="${OPTARG}"
         ;;
         L) changeStatelessStatus="${OPTARG}"
         ;;
         E) configReport="true"
         ;;
         *) Usage
         ;;
      esac
   done
fi

# Before we do anything, close any http fds we inherited
CloseHttpFileDescriptors

if [ "$configureAddress" = "true" ]; then
   if [ "$configureDHCP" = "on" -o "$configureDHCP" = "off" ]; then
      enableDHCP "$configureDHCP"
   elif [ "X$staticAddress" != "X" ]; then
      configureStaticAddress "$staticAddress"
   else
      Usage
   fi
elif [ "$getAddress" = "true" ]; then
   if [ "$operation" = "static" ]; then
      getStaticAddressInfo
   elif [ "$operation" = "all" ]; then
      getAllAddresses
   elif [ "$operation" = "host" ]; then
      getLibraryHostname
   elif [ "$operation" = "dns" ] ; then
      getDomainServers
   elif [ "$operation" = "ntp" ] ; then
      getNTPServers
   else
      Usage
   fi
elif [ "$removeStaticAddress" = "true" ]; then
   deleteStaticAddress "restart"
elif [ "$changeHostname" = "true" ]; then
   if [ "X$newHostname" = "X" ]; then
      Usage
   fi
   changeLibraryHostname $newHostname
elif [ -n "${domainServers}" ]; then
    configureDNS "${domainServers}"
elif [ -n "${ntpServers}" ]; then
    configureNTP "${ntpServers}"
elif [ "X${reportStatelessStatus}" = "Xtrue" ]; then
    isStatelessEnabled
elif [ "${changeStatelessStatus}" = "0" -o "${changeStatelessStatus}" = "1" ]; then
   enableStateless "${changeStatelessStatus}"
elif [ "$configReport" = "true" ]; then
   displayConfigReport
else
   Usage
fi

LogTrace "Exit status: $COMMAND_EXIT_STATUS"
exit $COMMAND_EXIT_STATUS
