// Javascript for creating form validations
// This script requires common.js

// You can cause the validation process to skip
// a field by disabling it, or by setting the
// property "novalidate" to "true".

function validateForm()
{
	// Loop through the associated fields and run
	// all of the validators on each.
	var errorString = "";
	var firstInvalidField;	

    var debugString = "";

	for(var i = 0; i < this.fields.length; i = i + 1)
	{
		var fieldName = this.fields[i];
		var validators = this.validators[i];
		
		debugString += (fieldName + ": ");
		
		// Get the field value
		var field = document.getElementById(fieldName);
		
		// Don't bother if the field is disabled, or has the
		// "novalidate" property set.
		if(field.disabled || field.novalidate)
		{
		    continue;
		}
		
		if(!field)
		{
			alert("Field could not be located: " + fieldName);
			continue;
		}
		
		var value = field.value;

		// Get the display name for the field

		var displayName = this.fieldNames[i];

		for(var v = 0; v < validators.length; v = v + 1)
		{
			validatorName = validators[v];

            debugString += (validatorName + ", ");

			// Lookup the validator
			var validIndex = this.knownValidatorNames.indexOf(validatorName);
			var validator = this.knownValidators[validIndex];
			var valid = validator.validate(value);

			// Was the validator successful?
			if(!valid)
			{
				if(!firstInvalidField)
				{
					firstInvalidField = field;
				}
				var rawError = validator.error;
				rawError = rawError.replace(/%name%/, displayName);
				if(errorString != "") errorString = errorString + "\n";
				errorString = errorString + "- " + rawError;
				validator.replace(value, field);
			}
		}
		
		debugString += "\n";
	}

    debugString += "\nGlobals: ";

	// Run the global validators - the value passed to these is ""
	
	for(var i = 0; i < this.globalValidators.length; i = i + 1)
	{
		var validator = this.globalValidators[i];
		var valid = validator.validate(value);

        debugString += (validator.name + ", ");

		// Was the validator successful?
		if(!valid)
		{
			if(!firstInvalidField)
			{
				firstInvalidField = field;
			}
			var rawError = validator.error;
			if(errorString != "") errorString = errorString + "\n";
			errorString = errorString + "- " + rawError;
			validator.replace("", "");
		}
	}

    //alert(debugString);

	if(errorString != "")
	{
		alert("There are errors on the page: \n\n" + errorString + "\n\nPlease fix these errors and try again.");
		transferFocus(firstInvalidField);
		return(false);	
	}
	return(true);
}

function associate(fieldName, validatorName, displayName)
{
	var currentSize = this.fields.length;
	
	var i = this.fields.indexOf(fieldName);
	if(i < 0)
	{
		// field is not yet known, so add it
		this.fields.length = currentSize + 1;
		this.fieldNames.length = currentSize + 1;
		this.validators.length = currentSize + 1;

		this.fields[currentSize] = fieldName;
		this.validators[currentSize] = new Array();
		
		// If a display name was given, set it in the names
		// array.  Otherwise, set the name property as
		// the display name.
		if(displayName)
		{
			this.fieldNames[currentSize] = displayName;
		}
		else
		{
			this.fieldNames[currentSize] = fieldName;
		}
		
		i = currentSize;
	}
	
	// Add the validator to the list
	var validatorListSize = this.validators[i].length;
	this.validators[i].length = validatorListSize + 1;
	this.validators[i][validatorListSize] = validatorName;
}

function registerValidator(validator)
{
	var currentSize = this.knownValidatorNames.length;
	this.knownValidatorNames.length = currentSize + 1;
	this.knownValidators.length = currentSize + 1;

	this.knownValidatorNames[currentSize] = validator.name;
	this.knownValidators[currentSize] = validator;
}

function runOnChange(fieldName)
{
	// OnChange seems to function the same as onBlur, at least
	// in FireFox.
	// Get the object
	var object = document.getElementById(fieldName);
	// Install a listener
	object.onchange = null;
	object.onchange = validateOnEvent;
	object.validationManager = this;
}

function runOnSubmit(formName)
{
	// Get the form
	var form = document.forms[formName];
	// Install a listener
	form.onsubmit = null;
	form.onsubmit = validateOnEvent;
	form.validationManager = this;
}

function validateOnEvent()
{
	var mgr = this.validationManager;
	return mgr.validate();
}

function installGlobalValidator(validator)
{
	var currentSize = this.globalValidators.length;
	this.globalValidators.length = currentSize + 1;

	this.globalValidators[currentSize] = validator;
}

// Create a validator for the form with the given name.
// You can instantiate this before the form because it
// will not attempt to locate the form until validate()
// is called.
function ValidatorManager()
{
	// Install Methods
	this.validate = validateForm;
	this.associate = associate;
	this.registerValidator = registerValidator;
	this.runOnSubmit = runOnSubmit;
	this.runOnChange = runOnChange;
	this.installGlobalValidator = installGlobalValidator;

	// Create properties
	// Lists of validated fields and their validators
	this.fields = new Array();
	this.fieldNames = new Array();
	this.validators = new Array();

	// These lists hold the validators that the object
	// knows how to use.  If you want to use a special
	// validator, you need to add it using 
	// registerValidator().
	this.knownValidatorNames = new Array();
	this.knownValidators = new Array();
	
	// Create list of special-form-wide validators.
	// these will have custom implementations for
	// getting the values to validate.
	this.globalValidators = new Array();
	
	this.registerValidator(new ValidatorAlpha());
	this.registerValidator(new ValidatorNumeric());
	this.registerValidator(new ValidatorNoHtml());
	this.registerValidator(new ValidatorRequired());
	this.registerValidator(new ValidatorIpv4());
	this.registerValidator(new ValidatorIpv6());
	this.registerValidator(new ValidatorDnsName());
	this.registerValidator(new ValidatorHostOrIp());
	this.registerValidator(new ValidatorEmail());
	this.registerValidator(new ValidatorBarcode());
	this.registerValidator(new ValidatorBarcodeFilter());
	this.registerValidator(new ValidatorTcpPort());
	this.registerValidator(new ValidatorUserName());
	this.registerValidator(new ValidatorPassword());
	this.registerValidator(new ValidatorMinute());
	this.registerValidator(new ValidatorHour());
}

function transferFocus(toField)
{
	focusField = toField;
	setTimeout( setTheFocus, 10 );
}

// -----------------------------------------------------
// Define validators here.  A validator needs to have
// two methods: validate() and replace().  replace()
// can be left blank if replacement behavior is not
// desireable.  It is also helpful to define the name
// as a global constant.  "name" and "error" properties
// should be defined as well.

// ----------------------------------------------------
// ALPHABETIC VALIDATOR
// ----------------------------------------------------
var VALID_ALPHA = "validAlpha";

function ValidatorAlpha()
{
	this.error = "%name% may contain only alphabetic characters";
	this.name = VALID_ALPHA;
}

ValidatorAlpha.prototype.validate = function(value)
{
	return !value.match(/[^a-zA-Z]/);
}
ValidatorAlpha.prototype.replace = function(value, field)
{
	field.value = value.replace(/[^a-zA-Z]/g, "");
}

// ----------------------------------------------------
// NUMERIC VALIDATOR
// ----------------------------------------------------

var VALID_NUMERIC = "validNumeric";

function ValidatorNumeric()
{
	this.error = "%name% may contain only numeric characters";
	this.name = VALID_NUMERIC;
}

ValidatorNumeric.prototype.validate = function(value)
{
	return !value.match(/\D/);
}
ValidatorNumeric.prototype.replace = function(value, field)
{
}

// ----------------------------------------------------
// USERNAME VALIDATOR
// ----------------------------------------------------

var VALID_USERNAME = "validUserName";

function ValidatorUserName()
{
	this.error = "%name% may contain only alpha-numeric characters or underscores";
	this.name = VALID_USERNAME;
}

ValidatorUserName.prototype.validate = function(value)
{
	return !value.match(/[^\da-zA-Z_]/);
}
ValidatorUserName.prototype.replace = function(value, field)
{
	field.value = value.replace(/[^\da-zA-Z_]/g, "");
}

// ----------------------------------------------------
// PASSWORD VALIDATOR
// ----------------------------------------------------

var VALID_PASSWORD = "validPassword";

function ValidatorPassword()
{
	this.error = "%name% may contain only lowercase letters, numbers, underscores, @, *, ., and -";
	this.name = VALID_PASSWORD;
}

ValidatorPassword.prototype.validate = function(value)
{
	return !value.match(/[^a-z0-9_@\.\-\*]/);
}
ValidatorPassword.prototype.replace = function(value, field)
{
	field.value = value.replace(/[^a-z0-9_@\.\-\*]/g, "");
}

// ----------------------------------------------------
// NO HTML VALIDATOR
// ----------------------------------------------------

var VALID_NO_HTML = "validNoHtml";

function ValidatorNoHtml()
{
	this.error = "%name% may contain only the following characters: alpha-numeric @ ( ) ; : / [ ] , . - _ * ?";
	this.name = VALID_NO_HTML;
}
ValidatorNoHtml.prototype.validate = function(value)
{
	return !value.match(/[^A-Za-z0-9@() ,;:\.\-_\[\]\/\*\?]/);
}
ValidatorNoHtml.prototype.replace = function(value, field)
{
	field.value = value.replace(/(<.*>)|(<!--)|(-->)|[^A-Za-z0-9@() ,;:\.\-_\[\]\/\*\?]/g, "");
}

// ----------------------------------------------------
// REQUIRED VALIDATOR
// "value" must not be empty, and must not be a sting
// composed entirely of whitespace.
// ----------------------------------------------------

var VALID_REQUIRED = "validRequired";

function ValidatorRequired()
{
	this.error = "%name% is a required field and cannot be empty or all white-space.";
	this.name = VALID_REQUIRED;
}
ValidatorRequired.prototype.validate = function(value)
{
	var noPattern = /^[ ]+$/;

	return value != "" && !value.match(noPattern);
}
ValidatorRequired.prototype.replace = function(value, field)
{
}

// ----------------------------------------------------
// IPV4 VALIDATOR
// ----------------------------------------------------

var VALID_IPV4 = "validIpv4";

function ValidatorIpv4()
{
	this.error = "%name% must be a valid IPv4 address.";
	this.name = VALID_IPV4;
}
ValidatorIpv4.prototype.validate = function(value)
{
	var re = /^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$/;

	if(value == "") return(true);

   if (re.test(value)) 
   {
      var parts = value.split(".");
      if (parseInt(parseFloat(parts[0])) == 0) 
      { 
         return(false); 
      }
      for (var i=0; i<parts.length; i++) 
      {
         if (parseInt(parseFloat(parts[i])) > 255) 
         { 
            return(false); 
         }
      }
      return(true);
   } 
   else 
   {
      return(false);
   }
}
ValidatorIpv4.prototype.replace = function(value, field)
{
}

// ----------------------------------------------------
// IPV6 VALIDATOR
// ----------------------------------------------------

var VALID_IPV6 = "validIpv6";

function ValidatorIpv6()
{
	this.error = "%name% must be a valid IPv6 address.";
	this.name = VALID_IPV6;
}
ValidatorIpv6.prototype.validate = function(value)
{
	var isSuppressed = suppressAlertsInIPV6;
	suppressAlertsInIPV6 = true;
	var valid = verifyIPv6NoAlert(value);
	if(!valid)
	{
		this.error = resultMsgIpv6;
	}
	suppressAlertsInIPV6 = isSuppressed;
	return valid;
}
ValidatorIpv6.prototype.replace = function(value, field)
{
}

// ----------------------------------------------------
// DNS VALIDATOR
// Must only contain letters, numbers, underscores &
// periods, and must have at least one letter, dash or
// underscore.
// ----------------------------------------------------

var VALID_DNS_NAME = "validDnsName";

function ValidatorDnsName()
{
	this.error = "%name% must be a valid host name and can contain only letters, numbers, underscores, dashes and periods.";
	this.name = VALID_DNS_NAME;
}
ValidatorDnsName.prototype.validate = function(value)
{
	var re = /[^A-Za-z0-9_\.-]/;
    var one_of_re = /[A-Za-z_-]/;

	if( value.match(re) || (value.length < 2 && value != ""))
	{
		return(false);
	}
	return(value.match(one_of_re));
}
ValidatorDnsName.prototype.replace = function(value, field)
{
}

// ----------------------------------------------------
// HOST/IPV4/IPV6 Validator
// This verifies that the value matches one of these
// patterns.
// ----------------------------------------------------

var VALID_HOST_OR_IP = "validHostOrIp";

function ValidatorHostOrIp()
{
	this.error = "%name% must be a valid host name, IPv4 or IPv6 address.";
	this.name = VALID_HOST_OR_IP;
	
	// Create pet validators for ipv4, ipv6 and dns name
	this.validV4 = new ValidatorIpv4();
	this.validV6 = new ValidatorIpv6();
	this.validDNS = new ValidatorDnsName();
}
ValidatorHostOrIp.prototype.validate = function(value)
{
    this.error = "%name% must be a valid host name, IPv4 or IPv6 address.";
    var result = true; // empty string should return true
    
	if(value && value.length > 0)
	{
	    // If we have ":", then validate as IPV6
	    if(value.indexOf(":") > -1)
	    {
	        result = this.validV6.validate(value);
	        
	        if(!result && resultMsgIpv6 != "") 
		    {
			    this.error = (this.error + " (" + resultMsgIpv6 + ")");
		    }
	    }
	    // Otherwise, see if it matches either the DNS name
	    // validator or the IPV4 validator.
	    else
	    {
	        result = this.validDNS.validate(value) || this.validV4.validate(value);
	    }
	}
	
	return result;
}
ValidatorHostOrIp.prototype.replace = function(value, field)
{
}

// ----------------------------------------------------
// EMAIL VALIDATOR
// ----------------------------------------------------

var VALID_EMAIL = "validEmail";

function ValidatorEmail()
{
	this.error = "%name% must be a valid e-mail address.";
	this.name = VALID_EMAIL;
}
ValidatorEmail.prototype.validate = function(value)
{
	AtPos = value.indexOf("@")
	StopPos = value.lastIndexOf(".")

	if (value != "" && ((AtPos == -1 || StopPos == -1) || (StopPos < AtPos) | (StopPos - AtPos == 1)))
	{
		return false;
	}

	return true;
}
ValidatorEmail.prototype.replace = function(value, field)
{
}

// ----------------------------------------------------
// BARCODE VALIDATOR
// ----------------------------------------------------

var VALID_BARCODE = "validBarcode";

function ValidatorBarcode()
{
	this.error = "%name% may contain only uppercase letters and numbers.";
	this.name = VALID_BARCODE;
}
ValidatorBarcode.prototype.validate = function(value)
{
	var pattern = /[^A-Z0-9]/;
	if(value.match(pattern))
	{
		return false;
	}
	return true;
}
ValidatorBarcode.prototype.replace = function(value, field)
{
	field.value = value.replace(/[^A-Z0-9]/g, "");
}

// ----------------------------------------------------
// BARCODE FILTER VALIDATOR
// ----------------------------------------------------

var VALID_BARCODE_FILTER = "validBarcodeFilter";

function ValidatorBarcodeFilter()
{
	this.error = "%name% may contain only uppercase letters and numbers.";
	this.name = VALID_BARCODE_FILTER;
}
ValidatorBarcodeFilter.prototype.validate = function(value)
{
   var pattern = /[^A-Za-z0-9*-]/;
   var multipleChar= /[*-]/g;
   var number;

   //remove leading and ending spaces in string
   value = value.replace(/^\s+/g, '').replace(/\s+$/g, '');

   // make sure all characters are valid.
   if( value.match(pattern) )
   {
      this.error = "%name%: Only letters, numbers, hyphens, and asterisks are allowed in the filter string.";
      return(false);
   }

   // no multiple *'s or -'s can be in a single string
   if (number=value.match(multipleChar))
   {
      if (number.length > 1)
      {
         this.error = "%name%: Only one asterisk or hyphen is allowed in a filter string.";
         return(false);
      }
   }

   return(true);
}
ValidatorBarcodeFilter.prototype.replace = function(value, field)
{
}

// ----------------------------------------------------
// TCP PORT VALIDATOR
// ----------------------------------------------------

var VALID_TCP_PORT = "validTcpPort";

function ValidatorTcpPort()
{
	this.error = "%name% must be a valid TCP port number from 1 to 65535.";
	this.name = VALID_TCP_PORT;
}
ValidatorTcpPort.prototype.validate = function(value)
{
	if( value != "" && (!parseInt(value) || (value >= 65535) || (value <= 0)))
	{
		return false;
	}

	return true;
}
ValidatorTcpPort.prototype.replace = function(value, field)
{
}

// ----------------------------------------------------
// HOUR VALIDATOR
// ----------------------------------------------------

var VALID_HOUR = "validHour";

function ValidatorHour()
{
	this.error = "%name% must be a number between 1 and 23.";
	this.name = VALID_HOUR;
}
ValidatorHour.prototype.validate = function(value)
{
	if( value != "" && ((value > 23) || (value < 0) || value.match(/\D/) || isNaN(value)))
	{
		return false;
	}

	return true;
}
ValidatorHour.prototype.replace = function(value, field)
{
	// replace anything thats not a number
	field.value = value.replace(/\D/g, "");
}

// ----------------------------------------------------
// MINUTE VALIDATOR
// ----------------------------------------------------

var VALID_MINUTE = "validMinute";

function ValidatorMinute()
{
	this.error = "%name% must be a number between 0 and 59.";
	this.name = VALID_MINUTE;
}
ValidatorMinute.prototype.validate = function(value)
{
	if( value != "" && ((value > 59) || (value < 0) || value.match(/\D/) || isNaN(value)))
	{
		return false;
	}

	return true;
}
ValidatorMinute.prototype.replace = function(value, field)
{
	// replace anything thats not a number
	field.value = value.replace(/\D/g, "");
}

// ----------------------------------------------------
// REQUIRED IF CHECKED
// Use this if you have a field that must be filled in
// if an associated checkbox is selected.
// ----------------------------------------------------

function ValidatorReqIfChecked(checkboxName, checkBoxDisplayName, installName)
{
	this.checkboxId = checkboxName;
	this.error = "If " + checkBoxDisplayName + " is selected, %name% must be filled in.";
	this.name = installName;
	// Create a pet required validator
	this.valreq = new ValidatorRequired();
}
ValidatorReqIfChecked.prototype.validate = function(value)
{
	checked = document.getElementById(this.checkboxId).checked;
	
	if(checked)
	{
		return this.valreq.validate(value);
	}

	return true;
}
ValidatorReqIfChecked.prototype.replace = function(value, field)
{
}

// ----------------------------------------------------
// REQUIRED IF NOT CHECKED
// Use this if you have a field that must be filled in
// if an associated checkbox is *not* selected.
// ----------------------------------------------------

function ValidatorReqIfNotChecked(checkboxName, checkBoxDisplayName, installName)
{
	this.checkboxId = checkboxName;
	this.error = "If " + checkBoxDisplayName + " is not selected, %name% must be filled in.";
	this.name = installName;
	// Create a pet required validator
	this.valreq = new ValidatorRequired();
}
ValidatorReqIfNotChecked.prototype.validate = function(value)
{
	checked = document.getElementById(this.checkboxId).checked;
	
	if(!checked)
	{
		return this.valreq.validate(value);
	}

	return true;
}
ValidatorReqIfNotChecked.prototype.replace = function(value, field)
{
}

// ----------------------------------------------------
// MUST BE DIFFERENT
// If two fields have the same value, false is returned
// ----------------------------------------------------

function ValidatorMustBeDifferent(otherFieldId, otherFieldDisplayName, installName)
{
	this.otherFieldId = otherFieldId;
	this.error = "%name% cannot have the same value as " + otherFieldDisplayName;
	this.name = installName;
}
ValidatorMustBeDifferent.prototype.validate = function(value)
{
	var otherValue = document.getElementById(this.otherFieldId).value;
	
	if(value == otherValue)
	{
		return false;
	}

	return true;
}
ValidatorMustBeDifferent.prototype.replace = function(value, field)
{
}
